"""
    Fetches audio and handles transcoding it for it.
"""


import sys
import tempfile
import subprocess
import os
import static_ffmpeg  # type: ignore

_PROCESS_TIMEOUT = 4 * 60 * 60


def _ytdlp_download(url: str, outdir: str) -> str:
    """Downloads a file using ytdlp."""
    os.makedirs(outdir, exist_ok=True)
    # remove all files in the directory
    for file in os.listdir(outdir):
        os.remove(os.path.join(outdir, file))
    cmd = f'yt-dlp --no-check-certificate {url} -o "out.%(ext)s"'
    subprocess.run(
        cmd,
        shell=True,
        cwd=outdir,
        check=True,
        timeout=_PROCESS_TIMEOUT,
        universal_newlines=True,
    )
    new_files = os.listdir(outdir)
    assert len(new_files) == 1, f"Expected 1 file, got {new_files}"
    downloaded_file = os.path.join(outdir, new_files[0])
    assert os.path.exists(
        downloaded_file
    ), f"The expected file {downloaded_file} doesn't exist"
    return downloaded_file


def _convert_to_mp3(inpath: str, outpath: str) -> None:
    """Converts a file to mp3."""
    cmd = f'ffmpeg -y -i "{inpath}" -acodec libmp3lame "{outpath}"'
    print(f"Running:\n  {cmd}")
    try:
        subprocess.run(
            cmd, shell=True, check=True, capture_output=True, timeout=_PROCESS_TIMEOUT
        )
    except subprocess.CalledProcessError as exc:
        print(f"Failed to run {cmd} with error {exc}")
        print(f"stdout: {exc.stdout}")
        print(f"stderr: {exc.stderr}")
        raise
    assert os.path.exists(outpath), f"The expected file {outpath} doesn't exist"


def fetch_audio(url_or_file: str, out_mp3: str) -> None:
    """Fetches from the internet or from a local file and outputs a wav file."""
    assert out_mp3.endswith(".mp3")
    static_ffmpeg.add_paths()  # pylint: disable=no-member
    if url_or_file.startswith("http") or url_or_file.startswith("ftp"):
        with tempfile.TemporaryDirectory() as tmpdir:
            print(f"Using temporary directory {tmpdir}")
            downloaded_file = _ytdlp_download(url_or_file, os.path.abspath(tmpdir))
            print("Downloaded file: ", downloaded_file)
            _convert_to_mp3(downloaded_file, out_mp3)
        sys.stderr.write("Downloading complete.\n")
        assert os.path.exists(out_mp3), f"The expected file {out_mp3} doesn't exist"
    else:
        assert os.path.isfile(url_or_file)
        cmd = f'ffmpeg -i "{url_or_file}" -acodec libmp3lame "{out_mp3}"'
        sys.stderr.write(f"Running:\n  {cmd}\n")
        subprocess.run(
            cmd, shell=True, check=True, capture_output=True, timeout=_PROCESS_TIMEOUT
        )
        assert os.path.exists(out_mp3), f"The expected file {out_mp3} doesn't exist"


def unit_test() -> None:
    """Runs the program."""
    url = "https://www.youtube.com/watch?v=8Wg8f2g_GQY"
    fetch_audio(url, "out.mp3")


if __name__ == "__main__":
    unit_test()



"""
    Api for using transcribe_anything from python. Allows bulk processing.
"""

# pylint: disable=too-many-arguments,broad-except,too-many-locals,unsupported-binary-operation,too-many-branches,too-many-statements

import os
import stat
import sys
import time
import subprocess
import shutil
from typing import Optional

from transcribe_anything.audio import fetch_audio
from transcribe_anything.util import (
    get_computing_device,
    sanitize_path,
    chop_double_extension,
)
from transcribe_anything.logger import log_error

PERMS = stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH | stat.S_IWOTH | stat.S_IWUSR | stat.S_IWGRP


def transcribe(
    url_or_file: str,
    output_dir: Optional[str] = None,
    model: Optional[str] = None,
    task: Optional[str] = None,
    language: Optional[str] = None,
    keep_audio: bool = False,
    device: Optional[str] = None,
    other_args: Optional[list[str]] = None,
) -> str:
    """
    Runs the program.
    """
    basename = os.path.basename(url_or_file)
    if not basename or basename == ".":  # if url_or_file is a directory
        # Defense against paths with a trailing /, for example:
        # https://example.com/, which will yield a basename of "".
        basename = os.path.basename(os.path.dirname(url_or_file))
        basename = sanitize_path(basename)
    if output_dir is None:
        if url_or_file.startswith("http"):
            # Try and the title of the video using yt-dlp
            # If that fails, use the basename of the url
            try:
                yt_dlp = subprocess.run(
                    ["yt-dlp", "--get-title", url_or_file],
                    capture_output=True,
                    text=True,
                    check=True,
                )
                output_dir = "text_" + yt_dlp.stdout.strip()
                output_dir = sanitize_path(output_dir[:80].strip())
            except Exception:
                log_error("yt-dlp failed to get title, using basename instead.")
                output_dir = basename
        else:
            output_dir = os.path.splitext(basename)[0]
    if os.path.exists(output_dir):
        shutil.rmtree(output_dir)
    os.makedirs(output_dir, exist_ok=True)
    tmp_mp3 = os.path.join(output_dir, "out.mp3")
    fetch_audio(url_or_file, tmp_mp3)
    assert os.path.exists(tmp_mp3), f"Path {tmp_mp3} doesn't exist."
    device = device or get_computing_device()
    if device == "cuda":
        print("#####################################")
        print("######### GPU ACCELERATED! ##########")
        print("#####################################")
    elif device == "cpu":
        print("WARNING: NOT using GPU acceleration, using 10x slower CPU instead.")
    else:
        raise ValueError(f"Unknown device {device}")
    print(f"Using device {device}")
    model_str = f" --model {model}" if model else ""
    output_dir_str = f' --output_dir "{output_dir}"' if output_dir else ""
    task_str = f" --task {task}" if task else ""
    language_str = f" --language {language}" if language else ""
    cmd_list = [
        "whisper",
        f'"{tmp_mp3}"',
        "--device",
        device,
        model_str,
        output_dir_str,
        task_str,
        language_str,
    ]
    if other_args:
        cmd_list.extend(other_args)
    # Remove the empty strings.
    cmd_list = [x.strip() for x in cmd_list if x.strip()]
    cmd = " ".join(cmd_list)
    sys.stderr.write(f"Running:\n  {cmd}\n")
    proc = subprocess.Popen(cmd, shell=True)  # pylint: disable=consider-using-with
    while True:
        rtn = proc.poll()
        if rtn is None:
            time.sleep(0.25)
            continue
        if rtn != 0:
            msg = f"Failed to execute {cmd}\n "
            raise OSError(msg)
        break
    files = [os.path.join(output_dir, name) for name in os.listdir(output_dir)]
    for file in files:
        # Change the filename to remove the double extension
        file_name = os.path.basename(file)
        base_path = os.path.dirname(file)
        new_file = os.path.join(base_path, chop_double_extension(file_name))
        if file != new_file:
            if os.path.exists(new_file):
                os.remove(new_file)
            os.rename(file, new_file)
    if not keep_audio:
        os.remove(tmp_mp3)
    return output_dir


if __name__ == "__main__":
    # test case for twitter video
    # transcribe(url_or_file="https://twitter.com/wlctv_ca/status/1598895698870951943")
    transcribe(url_or_file="https://www.youtube.com/watch?v=DWtpNPZ4tb4", output_dir="test")



"""
    Entry point for running the transcribe-anything prgram.
"""

import argparse
import sys

from transcribe_anything.api import transcribe
from transcribe_anything.util import get_computing_device
from transcribe_anything.parse_whisper_options import parse_whisper_options


def main() -> None:
    """Main entry point for the command line tool."""
    whisper_options = parse_whisper_options()
    device = get_computing_device()
    help_str = (
        f'transcribe_anything is using a "{device}" device.'
        " Any unrecognized args are assumed to be for whisper"
        " ai and will be passed as is to whisper ai."
    )
    parser = argparse.ArgumentParser(description=help_str)
    parser.add_argument(
        "url_or_file",
        help="Provide file path or url (includes youtube/facebook/twitter/etc)",
    )
    parser.add_argument(
        "--output_dir",
        help="Provide output directory name,d efaults to the filename of the file.",
        default=None,
    )
    parser.add_argument(
        "--model",
        help="name of the Whisper model to us",
        default="small",
        choices=whisper_options["model"],
    )
    parser.add_argument(
        "--task",
        help="whether to perform transcription or translation",
        default="transcribe",
        choices=whisper_options["task"],
    )
    parser.add_argument(
        "--language",
        help="language to the target audio is in, default None will auto-detect",
        default=None,
        choices=[None] + whisper_options["language"],
    )
    # keep_audio
    parser.add_argument(
        "--no-keep-audio",
        help="whether to keep the audio file after processing",
        default=False,
        action="store_true",
    )
    parser.add_argument(
        "--device",
        help="device to use for processing, None will auto select CUDA if available or else CPU",
        default=None,
        choices=[None, "cpu", "cuda"],
    )
    # add extra options that are passed into the transcribe function
    args, unknown = parser.parse_known_args()
    print(f"Unknown args: {unknown}")
    print(f"Running transcribe_audio on {args.url_or_file}")
    transcribe(
        url_or_file=args.url_or_file,
        output_dir=args.output_dir,
        model=args.model if args.model != "None" else None,
        task=args.task,
        language=args.language if args.language != "None" else None,
        keep_audio=not args.no_keep_audio,
        device=args.device,
        other_args=unknown,
    )


if __name__ == "__main__":
    # push sys argv prior to call
    sys.argv.append("test.mp3")
    sys.argv.append('--initial_prompt "What is your name?"')
    main()
    sys.exit(0)
